@@ -1,20 +1,24 @@
-# Copyright (c) 2009-2010 Arista Networks, Inc. - James Lingard
-# Copyright (c) 2004-2013 LOGILAB S.A. (Paris, FRANCE).
-# Copyright 2012 Google Inc.
-#
-# http://www.logilab.fr/ -- mailto:contact@logilab.fr
-# This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
-# Foundation; either version 2 of the License, or (at your option) any later
-# version.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details
-#
-# You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# -*- coding: utf-8 -*-
+# Copyright (c) 2009 Charles Hebert <charles.hebert@logilab.fr>
+# Copyright (c) 2010-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
+# Copyright (c) 2010 Daniel Harding <dharding@gmail.com>
+# Copyright (c) 2012-2014 Google, Inc.
+# Copyright (c) 2013-2018 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2014 Brett Cannon <brett@python.org>
+# Copyright (c) 2014 Arun Persaud <arun@nubati.net>
+# Copyright (c) 2015 Rene Zhang <rz99@cornell.edu>
+# Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
+# Copyright (c) 2016, 2018 Jakub Wilk <jwilk@jwilk.net>
+# Copyright (c) 2016 Peter Dawyndt <Peter.Dawyndt@UGent.be>
+# Copyright (c) 2017 Łukasz Rogalski <rogalski.91@gmail.com>
+# Copyright (c) 2017 Ville Skyttä <ville.skytta@iki.fi>
+# Copyright (c) 2018 Nick Drozd <nicholasdrozd@gmail.com>
+# Copyright (c) 2018 Anthony Sottile <asottile@umich.edu>
+
+
+# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+# For details: https://github.com/PyCQA/pylint/blob/master/COPYING
+
 """Checker for string formatting operations.
 """
 
@@ -23,8 +27,6 @@ import tokenize
 import string
 import numbers
 
-import six
-
 import astroid
 from pylint.interfaces import ITokenChecker, IAstroidChecker, IRawChecker
 from pylint.checkers import BaseChecker, BaseTokenChecker
@@ -38,7 +40,7 @@ _PY27 = sys.version_info[:2] == (2, 7)
 MSGS = {
     'E1300': ("Unsupported format character %r (%#02x) at index %d",
               "bad-format-character",
-              "Used when a unsupported format character is used in a format\
+              "Used when an unsupported format character is used in a format\
               string."),
     'E1301': ("Format string ends in middle of conversion specifier",
               "truncated-format-string",
@@ -61,7 +63,7 @@ MSGS = {
     'W1301': ("Unused key %r in format string dictionary",
               "unused-format-string-key",
               "Used when a format string that uses named conversion specifiers \
-              is used with a dictionary that conWtains keys not required by the \
+              is used with a dictionary that contains keys not required by the \
               format string."),
     'E1304': ("Missing key %r in format string dictionary",
               "missing-format-string-key",
@@ -76,7 +78,10 @@ MSGS = {
               "too-few-format-args",
               "Used when a format string that uses unnamed conversion \
               specifiers is given too few arguments"),
-
+    'E1310': ("Suspicious argument in %s.%s call",
+              "bad-str-strip-call",
+              "The argument to a str.{l,r,}strip call contains a"
+              " duplicate character, "),
     'W1302': ("Invalid format string",
               "bad-format-string",
               "Used when a PEP 3101 format string is invalid.",
@@ -95,7 +100,7 @@ MSGS = {
     'W1305': ("Format string contains both automatic field numbering "
               "and manual field specification",
               "format-combined-specification",
-              "Usen when a PEP 3101 format string contains both automatic "
+              "Used when a PEP 3101 format string contains both automatic "
               "field numbering (e.g. '{}') and manual field "
               "specification (e.g. '{0}').",
               {'minversion': (2, 7)}),
@@ -113,7 +118,7 @@ MSGS = {
               {'minversion': (2, 7)})
     }
 
-OTHER_NODES = (astroid.Const, astroid.List, astroid.Repr,
+OTHER_NODES = (astroid.Const, astroid.List,
                astroid.Lambda, astroid.FunctionDef,
                astroid.ListComp, astroid.SetComp, astroid.GeneratorExp)
 
@@ -121,7 +126,11 @@ if _PY3K:
     import _string # pylint: disable=wrong-import-position, wrong-import-order
 
     def split_format_field_names(format_string):
-        return _string.formatter_field_name_split(format_string)
+        try:
+            return _string.formatter_field_name_split(format_string)
+        except ValueError:
+            raise utils.IncompleteFormatString()
+
 else:
     def _field_iterator_convertor(iterator):
         for is_attr, key in iterator:
@@ -131,7 +140,10 @@ else:
                 yield is_attr, key
 
     def split_format_field_names(format_string):
-        keyname, fielditerator = format_string._formatter_field_name_split()
+        try:
+            keyname, fielditerator = format_string._formatter_field_name_split()
+        except ValueError:
+            raise utils.IncompleteFormatString
         # it will return longs, instead of ints, which will complicate
         # the output
         return keyname, _field_iterator_convertor(fielditerator)
@@ -191,24 +203,27 @@ def parse_format_method_string(format_string):
                 # to different output between 2 and 3
                 manual_pos_arg.add(str(keyname))
                 keyname = int(keyname)
-            keys.append((keyname, list(fielditerator)))
+            try:
+                keys.append((keyname, list(fielditerator)))
+            except ValueError:
+                raise utils.IncompleteFormatString()
         else:
             num_args += 1
     return keys, num_args, len(manual_pos_arg)
 
-def get_args(callfunc):
-    """Get the arguments from the given `CallFunc` node.
+def get_args(call):
+    """Get the arguments from the given `Call` node.
 
     Return a tuple, where the first element is the
     number of positional arguments and the second element
     is the keyword arguments in a dict.
     """
-    if callfunc.keywords:
+    if call.keywords:
         named = {arg.arg: utils.safe_infer(arg.value)
-                 for arg in callfunc.keywords}
+                 for arg in call.keywords}
     else:
         named = {}
-    positional = len(callfunc.args)
+    positional = len(call.args)
     return positional, named
 
 def get_access_path(key, parts):
@@ -241,7 +256,7 @@ class StringFormatChecker(BaseChecker):
         args = node.right
 
         if not (isinstance(left, astroid.Const)
-                and isinstance(left.value, six.string_types)):
+                and isinstance(left.value, str)):
             return
         format_string = left.value
         try:
@@ -270,7 +285,7 @@ class StringFormatChecker(BaseChecker):
                 for k, _ in args.items:
                     if isinstance(k, astroid.Const):
                         key = k.value
-                        if isinstance(key, six.string_types):
+                        if isinstance(key, str):
                             keys.add(key)
                         else:
                             self.add_message('bad-format-string-key',
@@ -278,7 +293,7 @@ class StringFormatChecker(BaseChecker):
                     else:
                         # One of the keys was something other than a
                         # constant.  Since we can't tell what it is,
-                        # supress checks for missing keys in the
+                        # suppress checks for missing keys in the
                         # dictionary.
                         unknown_keys = True
                 if not unknown_keys:
@@ -304,7 +319,10 @@ class StringFormatChecker(BaseChecker):
             # the % operator matches the number required by the format
             # string.
             if isinstance(args, astroid.Tuple):
-                num_args = len(args.elts)
+                rhs_tuple = utils.safe_infer(args)
+                num_args = None
+                if rhs_tuple not in (None, astroid.Uninferable):
+                    num_args = len(rhs_tuple.elts)
             elif isinstance(args, OTHER_NODES + (astroid.Dict, astroid.DictComp)):
                 num_args = 1
             else:
@@ -319,16 +337,6 @@ class StringFormatChecker(BaseChecker):
                     self.add_message('too-few-format-args', node=node)
 
 
-class StringMethodsChecker(BaseChecker):
-    __implements__ = (IAstroidChecker,)
-    name = 'string'
-    msgs = {
-        'E1310': ("Suspicious argument in %s.%s call",
-                  "bad-str-strip-call",
-                  "The argument to a str.{l,r,}strip call contains a"
-                  " duplicate character, "),
-        }
-
     @check_messages(*(MSGS.keys()))
     def visit_call(self, node):
         func = utils.safe_infer(node.func)
@@ -368,7 +376,7 @@ class StringMethodsChecker(BaseChecker):
             return
         if not isinstance(strnode, astroid.Const):
             return
-        if not isinstance(strnode.value, six.string_types):
+        if not isinstance(strnode.value, str):
             return
 
         if node.starargs or node.kwargs:
@@ -383,8 +391,8 @@ class StringMethodsChecker(BaseChecker):
             self.add_message('bad-format-string', node=node)
             return
 
-        named_fields = set(field[0] for field in fields
-                           if isinstance(field[0], six.string_types))
+        named_fields = {field[0] for field in fields
+                        if isinstance(field[0], str)}
         if num_args and manual_pos:
             self.add_message('format-combined-specification',
                              node=node)
@@ -451,13 +459,13 @@ class StringMethodsChecker(BaseChecker):
                 if key not in named:
                     continue
                 argname = named[key]
-            if argname in (astroid.YES, None):
+            if argname in (astroid.Uninferable, None):
                 continue
             try:
                 argument = next(argname.infer())
             except astroid.InferenceError:
                 continue
-            if not specifiers or argument is astroid.YES:
+            if not specifiers or argument is astroid.Uninferable:
                 # No need to check this key if it doesn't
                 # use attribute / item access
                 continue
@@ -468,7 +476,7 @@ class StringMethodsChecker(BaseChecker):
             previous = argument
             parsed = []
             for is_attribute, specifier in specifiers:
-                if previous is astroid.YES:
+                if previous is astroid.Uninferable:
                     break
                 parsed.append((is_attribute, specifier))
                 if is_attribute:
@@ -488,11 +496,15 @@ class StringMethodsChecker(BaseChecker):
                     warn_error = False
                     if hasattr(previous, 'getitem'):
                         try:
-                            previous = previous.getitem(specifier)
-                        except (IndexError, TypeError):
+                            previous = previous.getitem(astroid.Const(specifier))
+                        except (astroid.AstroidIndexError,
+                                astroid.AstroidTypeError,
+                                astroid.AttributeInferenceError):
                             warn_error = True
                         except astroid.InferenceError:
                             break
+                        if previous is astroid.Uninferable:
+                            break
                     else:
                         try:
                             # Lookup __getitem__ in the current node,
@@ -516,7 +528,6 @@ class StringMethodsChecker(BaseChecker):
                     break
 
 
-
 class StringConstantChecker(BaseTokenChecker):
     """Check string literals"""
     __implements__ = (ITokenChecker, IRawChecker)
@@ -622,5 +633,4 @@ class StringConstantChecker(BaseTokenChecker):
 def register(linter):
     """required method to auto register this checker """
     linter.register_checker(StringFormatChecker(linter))
-    linter.register_checker(StringMethodsChecker(linter))
     linter.register_checker(StringConstantChecker(linter))
