@@ -1,34 +1,54 @@
 # -*- coding: utf-8 -*-
 # Copyright (c) 2006-2015 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
 # Copyright (c) 2008 Fabrice Douchant <Fabrice.Douchant@logilab.fr>
-# Copyright (c) 2009 Mads Kiilerich <mads@kiilerich.com>
 # Copyright (c) 2009 Vincent
+# Copyright (c) 2009 Mads Kiilerich <mads@kiilerich.com>
 # Copyright (c) 2011-2014 Google, Inc.
-# Copyright (c) 2012 Kevin Jing Qiu <kevin.jing.qiu@gmail.com>
 # Copyright (c) 2012 David Pursehouse <david.pursehouse@sonymobile.com>
-# Copyright (c) 2012 JT Olds <jtolds@xnet5.com>
+# Copyright (c) 2012 Kevin Jing Qiu <kevin.jing.qiu@gmail.com>
 # Copyright (c) 2012 FELD Boris <lothiraldan@gmail.com>
+# Copyright (c) 2012 JT Olds <jtolds@xnet5.com>
+# Copyright (c) 2014-2018 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2014-2015 Michal Nowikowski <godfryd@gmail.com>
 # Copyright (c) 2014 Brett Cannon <brett@python.org>
-# Copyright (c) 2014-2016 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2014 Alexandru Coman <fcoman@bitdefender.com>
+# Copyright (c) 2014 Daniel Harding <dharding@living180.net>
 # Copyright (c) 2014 Arun Persaud <arun@nubati.net>
 # Copyright (c) 2014 Dan Goldsmith <djgoldsmith@googlemail.com>
-# Copyright (c) 2014 Daniel Harding <dharding@living180.net>
-# Copyright (c) 2014-2015 Michal Nowikowski <godfryd@gmail.com>
-# Copyright (c) 2014 Alexandru Coman <fcoman@bitdefender.com>
+# Copyright (c) 2015-2016 Florian Bruhin <me@the-compiler.org>
 # Copyright (c) 2015 Aru Sahni <arusahni@gmail.com>
-# Copyright (c) 2015 Simu Toni <simutoni@gmail.com>
 # Copyright (c) 2015 Steven Myint <hg@stevenmyint.com>
-# Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
-# Copyright (c) 2015 Florian Bruhin <me@the-compiler.org>
+# Copyright (c) 2015 Simu Toni <simutoni@gmail.com>
 # Copyright (c) 2015 Mihai Balint <balint.mihai@gmail.com>
+# Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
+# Copyright (c) 2016-2017 Łukasz Rogalski <rogalski.91@gmail.com>
+# Copyright (c) 2016 Glenn Matthews <glenn@e-dad.net>
 # Copyright (c) 2016 Alan Evangelista <alanoe@linux.vnet.ibm.com>
+# Copyright (c) 2017-2018 Ville Skyttä <ville.skytta@iki.fi>
+# Copyright (c) 2017-2018 hippo91 <guillaume.peillex@gmail.com>
+# Copyright (c) 2017 Daniel Miller <millerdev@gmail.com>
+# Copyright (c) 2017 Roman Ivanov <me@roivanov.com>
+# Copyright (c) 2017 Ned Batchelder <ned@nedbatchelder.com>
+# Copyright (c) 2018 Randall Leeds <randall@bleeds.info>
+# Copyright (c) 2018 Mike Frysinger <vapier@gmail.com>
+# Copyright (c) 2018 ssolanki <sushobhitsolanki@gmail.com>
+# Copyright (c) 2018 Ville Skyttä <ville.skytta@upcloud.com>
+# Copyright (c) 2018 Sushobhit <31987769+sushobhit27@users.noreply.github.com>
+# Copyright (c) 2018 Anthony Sottile <asottile@umich.edu>
+# Copyright (c) 2018 Jason Owen <jason.a.owen@gmail.com>
+# Copyright (c) 2018 Gary Tyler McLeod <mail@garytyler.com>
+# Copyright (c) 2018 Yuval Langer <yuvallanger@mail.tau.ac.il>
+# Copyright (c) 2018 Nick Drozd <nicholasdrozd@gmail.com>
+# Copyright (c) 2018 kapsh <kapsh@kap.sh>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
 
-""" %prog [options] module_or_package
+# pylint: disable=broad-except
 
-  Check that a module satisfies a coding standard (and more !).
+""" %prog [options] modules_or_packages
+
+  Check that module(s) satisfy a coding standard (and more !).
 
     %prog --help
 
@@ -49,10 +69,8 @@ try:
 except ImportError:
     multiprocessing = None
 import sys
-import warnings
 import tokenize
-
-import six
+import warnings
 
 import astroid
 from astroid.__pkginfo__ import version as astroid_version
@@ -60,6 +78,7 @@ from astroid import modutils
 from pylint import checkers
 from pylint import interfaces
 from pylint import reporters
+from pylint import exceptions
 from pylint import utils
 from pylint import config
 from pylint.__pkginfo__ import version
@@ -67,10 +86,7 @@ from pylint.reporters.ureports import nodes as report_nodes
 
 
 MANAGER = astroid.MANAGER
-INCLUDE_IDS_HELP = ("Deprecated. It was used to include message\'s "
-                    "id in output. Use --msg-template instead.")
-SYMBOLS_HELP = ("Deprecated. It was used to include symbolic ids of "
-                "messages in output. Use --msg-template instead.")
+
 
 def _get_new_args(message):
     location = (
@@ -100,6 +116,7 @@ def _get_python_path(filepath):
         dirname = os.path.dirname(dirname)
         if old_dirname == dirname:
             return os.getcwd()
+    return None
 
 
 def _merge_stats(stats):
@@ -109,7 +126,7 @@ def _merge_stats(stats):
         message_stats = stat.pop('by_msg', {})
         by_msg.update(message_stats)
 
-        for key, item in six.iteritems(stat):
+        for key, item in stat.items():
             if key not in merged:
                 merged[key] = item
             else:
@@ -154,7 +171,7 @@ MSGS = {
               'traceback. Please report such errors !'),
     'F0010': ('error while code parsing: %s',
               'parse-error',
-              'Used when an exception occured while building the Astroid '
+              'Used when an exception occurred while building the Astroid '
               'representation which could be handled by astroid.'),
 
     'I0001': ('Unable to run raw checkers on built-in module %s',
@@ -221,8 +238,8 @@ if multiprocessing is not None:
 
             # Run linter for received files/modules.
             for file_or_module in iter(tasks_queue.get, 'STOP'):
-                result = self._run_linter(file_or_module[0])
                 try:
+                    result = self._run_linter(file_or_module[0])
                     results_queue.put(result)
                 except Exception as ex:
                     print("internal error with sending report for module %s" %
@@ -257,6 +274,7 @@ if multiprocessing is not None:
                     msgs, linter.stats, linter.msg_status)
 
 
+# pylint: disable=too-many-instance-attributes
 class PyLinter(config.OptionsManagerMixIn,
                utils.MessagesHandlerMixIn,
                utils.ReportsHandlerMixIn,
@@ -269,8 +287,8 @@ class PyLinter(config.OptionsManagerMixIn,
     * handle message activation / deactivation at the module level
     * handle some basic but necessary stats'data (number of classes, methods...)
 
-    IDE plugins developpers: you may have to call
-    `astroid.builder.MANAGER.astroid_cache.clear()` accross run if you want
+    IDE plugin developers: you may have to call
+    `astroid.builder.MANAGER.astroid_cache.clear()` across runs if you want
     to ensure the latest code version is actually checked.
     """
 
@@ -312,25 +330,16 @@ class PyLinter(config.OptionsManagerMixIn,
                   'short': 'f',
                   'group': 'Reports',
                   'help' : 'Set the output format. Available formats are text,'
-                           ' parseable, colorized, msvs (visual studio) and html. You '
-                           'can also give a reporter class, eg mypackage.mymodule.'
+                           ' parseable, colorized, json and msvs (visual studio).'
+                           ' You can also give a reporter class, e.g. mypackage.mymodule.'
                            'MyReporterClass.'}),
 
-                ('files-output',
-                 {'default': 0, 'type' : 'yn', 'metavar' : '<y_or_n>',
-                  'group': 'Reports', 'level': 1,
-                  'help' : 'Put messages in a separate file for each module / '
-                           'package specified on the command line instead of printing '
-                           'them on stdout. Reports (if any) will be written in a file '
-                           'name "pylint_global.[txt|html]". This option is deprecated and '
-                           'it will be removed in Pylint 2.0.'}),
-
                 ('reports',
-                 {'default': 1, 'type' : 'yn', 'metavar' : '<y_or_n>',
+                 {'default': False, 'type' : 'yn', 'metavar' : '<y_or_n>',
                   'short': 'r',
                   'group': 'Reports',
                   'help' : 'Tells whether to display a full report or only the '
-                           'messages'}),
+                           'messages.'}),
 
                 ('evaluation',
                  {'type' : 'string', 'metavar' : '<python_expression>',
@@ -344,8 +353,11 @@ class PyLinter(config.OptionsManagerMixIn,
                            'warnings messages and the total number of '
                            'statements analyzed. This is used by the global '
                            'evaluation report (RP0004).'}),
-
-                ('comment', utils.deprecated_option(opt_type='yn')),
+                ('score',
+                 {'default': True, 'type': 'yn', 'metavar': '<y_or_n>',
+                  'short': 's',
+                  'group': 'Reports',
+                  'help': 'Activate the evaluation score.'}),
 
                 ('confidence',
                  {'type' : 'multiple_choice', 'metavar': '<levels>',
@@ -353,7 +365,7 @@ class PyLinter(config.OptionsManagerMixIn,
                   'choices': [c.name for c in interfaces.CONFIDENCE_LEVELS],
                   'group': 'Messages control',
                   'help' : 'Only show warnings with the listed confidence levels.'
-                           ' Leave empty to show all. Valid levels: %s' % (
+                           ' Leave empty to show all. Valid levels: %s.' % (
                                ', '.join(c.name for c in interfaces.CONFIDENCE_LEVELS),)}),
 
                 ('enable',
@@ -365,24 +377,24 @@ class PyLinter(config.OptionsManagerMixIn,
                            'separated by comma (,) or put this option multiple time '
                            '(only on the command line, not in the configuration file '
                            'where it should appear only once). '
-                           'See also the "--disable" option for examples. '}),
+                           'See also the "--disable" option for examples.'}),
 
                 ('disable',
                  {'type' : 'csv', 'metavar': '<msg ids>',
                   'short': 'd',
                   'group': 'Messages control',
                   'help' : 'Disable the message, report, category or checker '
-                           'with the given id(s). You can either give multiple identifiers'
-                           ' separated by comma (,) or put this option multiple times '
+                           'with the given id(s). You can either give multiple identifiers '
+                           'separated by comma (,) or put this option multiple times '
                            '(only on the command line, not in the configuration file '
-                           'where it should appear only once).'
+                           'where it should appear only once). '
                            'You can also use "--disable=all" to disable everything first '
                            'and then reenable specific checks. For example, if you want '
                            'to run only the similarities checker, you can use '
                            '"--disable=all --enable=similarities". '
                            'If you want to run only the classes checker, but have no '
-                           'Warning level messages displayed, use'
-                           '"--disable=all --enable=classes --disable=W"'}),
+                           'Warning level messages displayed, use '
+                           '"--disable=all --enable=classes --disable=W".'}),
 
                 ('msg-template',
                  {'type' : 'string', 'metavar': '<template>',
@@ -390,14 +402,15 @@ class PyLinter(config.OptionsManagerMixIn,
                   'help' : ('Template used to display messages. '
                             'This is a python new-style format string '
                             'used to format the message information. '
-                            'See doc for all details')
+                            'See doc for all details.')
                  }),
 
                 ('jobs',
                  {'type' : 'int', 'metavar': '<n-processes>',
                   'short': 'j',
                   'default': 1,
-                  'help' : '''Use multiple processes to speed up Pylint.''',
+                  'help' : 'Use multiple processes to speed up Pylint. Specifying 0 will '
+                           'auto-detect the number of processors available to use.',
                  }),
 
                 ('unsafe-load-any-extension',
@@ -406,32 +419,34 @@ class PyLinter(config.OptionsManagerMixIn,
                            ' are imported into the active Python interpreter and'
                            ' may run arbitrary code.')}),
 
+                ('limit-inference-results',
+                 {'type': 'int', 'metavar': '<number-of-results>', 'default': 100,
+                  'help': ('Control the amount of potential inferred values when inferring '
+                           'a single object. This can help the performance when dealing with '
+                           'large functions or complex, nested conditions. ')}),
+
                 ('extension-pkg-whitelist',
                  {'type': 'csv', 'metavar': '<pkg[,pkg]>', 'default': [],
                   'help': ('A comma-separated list of package or module names'
                            ' from where C extensions may be loaded. Extensions are'
                            ' loading into the active Python interpreter and may run'
-                           ' arbitrary code')}
-                ),
+                           ' arbitrary code.')}),
+                ('suggestion-mode',
+                 {'type': 'yn', 'metavar': '<yn>', 'default': True,
+                  'help': ('When enabled, pylint would attempt to guess common '
+                           'misconfiguration and emit user-friendly hints instead '
+                           'of false-positive error messages.')}),
 
-                ('optimize-ast',
-                 {'type': 'yn', 'metavar': '<yn>', 'default': False,
-                  'help': ('Allow optimization of some AST trees. This will '
-                           'activate a peephole AST optimizer, which will '
-                           'apply various small optimizations. For instance, '
-                           'it can be used to obtain the result of joining '
-                           'multiple strings with the addition operator. '
-                           'Joining a lot of strings can lead to a maximum '
-                           'recursion error in Pylint and this flag can prevent '
-                           'that. It has one side effect, the resulting AST '
-                           'will be different than the one from reality. '
-                           'This option is deprecated and it will be removed in Pylint 2.0.')}
-                ),
+                ('exit-zero',
+                 {'action': 'store_true',
+                  'help': ('Always return a 0 (non-error) status code, even if '
+                           'lint errors are found. This is primarily useful in '
+                           'continuous integration scripts.')}),
                )
 
     option_groups = (
-        ('Messages control', 'Options controling analysis messages'),
-        ('Reports', 'Options related to output formating and reporting'),
+        ('Messages control', 'Options controlling analysis messages'),
+        ('Reports', 'Options related to output formatting and reporting'),
         )
 
     def __init__(self, options=(), reporter=None, option_groups=(),
@@ -460,7 +475,7 @@ class PyLinter(config.OptionsManagerMixIn,
             'disable': self.disable}
         self._bw_options_methods = {'disable-msg': self.disable,
                                     'enable-msg': self.enable}
-        full_version = '%%prog %s, \nastroid %s\nPython %s' % (
+        full_version = '%%prog %s\nastroid %s\nPython %s' % (
             version, astroid_version, sys.version)
         utils.MessagesHandlerMixIn.__init__(self)
         utils.ReportsHandlerMixIn.__init__(self)
@@ -476,8 +491,6 @@ class PyLinter(config.OptionsManagerMixIn,
                          report_messages_by_module_stats),
                         ('RP0003', 'Messages',
                          report_messages_stats),
-                        ('RP0004', 'Global evaluation',
-                         self.report_evaluation),
                        )
         self.register_checker(self)
         self._dynamic_plugins = set()
@@ -511,12 +524,20 @@ class PyLinter(config.OptionsManagerMixIn,
         if name in self._reporters:
             self.set_reporter(self._reporters[name]())
         else:
-            qname = self._reporter_name
-            module = modutils.load_module_from_name(
-                modutils.get_module_part(qname))
-            class_name = qname.split('.')[-1]
-            reporter_class = getattr(module, class_name)
-            self.set_reporter(reporter_class())
+            try:
+                reporter_class = self._load_reporter_class()
+            except (ImportError, AttributeError):
+                raise exceptions.InvalidReporterError(name)
+            else:
+                self.set_reporter(reporter_class())
+
+    def _load_reporter_class(self):
+        qname = self._reporter_name
+        module = modutils.load_module_from_name(
+            modutils.get_module_part(qname))
+        class_name = qname.split('.')[-1]
+        reporter_class = getattr(module, class_name)
+        return reporter_class
 
     def set_reporter(self, reporter):
         """set the reporter used to display messages and reports"""
@@ -595,7 +616,7 @@ class PyLinter(config.OptionsManagerMixIn,
             self.disable(checker.name)
 
     def disable_noerror_messages(self):
-        for msgcat, msgids in six.iteritems(self.msgs_store._msgs_by_category):
+        for msgcat, msgids in self.msgs_store._msgs_by_category.items():
             # enable only messages with 'error' severity and above ('fatal')
             if msgcat in ['E', 'F']:
                 for msgid in msgids:
@@ -606,7 +627,7 @@ class PyLinter(config.OptionsManagerMixIn,
 
     def disable_reporters(self):
         """disable all reporters"""
-        for _reporters in six.itervalues(self._reports):
+        for _reporters in self._reports.values():
             for report_id, _, _ in _reporters:
                 self.disable_report(report_id)
 
@@ -620,10 +641,15 @@ class PyLinter(config.OptionsManagerMixIn,
             for msg_id in self._checker_messages('python3'):
                 if msg_id.startswith('E'):
                     self.enable(msg_id)
+            config_parser = self.cfgfile_parser
+            if config_parser.has_option('MESSAGES CONTROL', 'disable'):
+                value = config_parser.get('MESSAGES CONTROL', 'disable')
+                self.global_set_option('disable', value)
         else:
             self.disable('python3')
         self.set_option('reports', False)
         self.set_option('persistent', False)
+        self.set_option('score', False)
 
     def python3_porting_mode(self):
         """Disable all other checkers and enable Python 3 warnings."""
@@ -638,6 +664,10 @@ class PyLinter(config.OptionsManagerMixIn,
                     self.enable(msg_id)
                 else:
                     self.disable(msg_id)
+        config_parser = self.cfgfile_parser
+        if config_parser.has_option('MESSAGES CONTROL', 'disable'):
+            value = config_parser.get('MESSAGES CONTROL', 'disable')
+            self.global_set_option('disable', value)
         self._python3_porting_mode = True
 
     # block level option handling #############################################
@@ -675,7 +705,7 @@ class PyLinter(config.OptionsManagerMixIn,
                     meth = self._options_methods[opt]
                 except KeyError:
                     meth = self._bw_options_methods[opt]
-                    # found a "(dis|en)able-msg" pragma deprecated suppresssion
+                    # found a "(dis|en)able-msg" pragma deprecated suppression
                     self.add_message('deprecated-pragma', line=start[0],
                                      args=(opt, opt.replace('-msg', '')))
                 for msgid in utils._splitstrip(value):
@@ -690,8 +720,16 @@ class PyLinter(config.OptionsManagerMixIn,
                             self.add_message('file-ignored', line=start[0])
                             self._ignore_file = True
                             return
-                        meth(msgid, 'module', start[0])
-                    except utils.UnknownMessage:
+                        comments_sharp_sep = content.split('#')[1:]
+                        first_comment = "#" + comments_sharp_sep[0]
+                        first_comment_match_disable = utils.OPTION_RGX.search(first_comment)
+                        # Deactivate msg emission for whole module only if
+                        # we are sure the disable directive is the first comment.
+                        # If not then it refers to the comment before
+                        # and not to the module itself.
+                        if first_comment_match_disable:
+                            meth(msgid, 'module', start[0])
+                    except exceptions.UnknownMessageError:
                         self.add_message('bad-option-value', args=msgid, line=start[0])
             else:
                 self.add_message('unrecognized-inline-option', args=opt, line=start[0])
@@ -701,7 +739,7 @@ class PyLinter(config.OptionsManagerMixIn,
 
     def get_checkers(self):
         """return all available checkers as a list"""
-        return [self] + [c for _checkers in six.itervalues(self._checkers)
+        return [self] + [c for _checkers in self._checkers.values()
                          for c in _checkers if c is not self]
 
     def prepare_checkers(self):
@@ -711,8 +749,8 @@ class PyLinter(config.OptionsManagerMixIn,
         # get needed checkers
         neededcheckers = [self]
         for checker in self.get_checkers()[1:]:
-            messages = set(msg for msg in checker.msgs
-                           if self.is_message_enabled(msg))
+            messages = {msg for msg in checker.msgs
+                        if self.is_message_enabled(msg)}
             if (messages or
                     any(self.report_is_enabled(r[0]) for r in checker.reports)):
                 neededcheckers.append(checker)
@@ -722,7 +760,9 @@ class PyLinter(config.OptionsManagerMixIn,
                                 reverse=True)
         return neededcheckers
 
-    def should_analyze_file(self, modname, path): # pylint: disable=unused-argument, no-self-use
+    # pylint: disable=unused-argument
+    @staticmethod
+    def should_analyze_file(modname, path, is_argument=False):
         """Returns whether or not a module should be checked.
 
         This implementation returns True for all python source file, indicating
@@ -733,10 +773,16 @@ class PyLinter(config.OptionsManagerMixIn,
 
         :param str modname: The name of the module to be checked.
         :param str path: The full path to the source code of the module.
+        :param bool is_argument: Whetter the file is an argument to pylint or not.
+                                 Files which respect this property are always
+                                 checked, since the user requested it explicitly.
         :returns: True if the module should be checked.
         :rtype: bool
         """
+        if is_argument:
+            return True
         return path.endswith('.py')
+    # pylint: enable=unused-argument
 
     def check(self, files_or_modules):
         """main checking entry: check a list of files or modules from their
@@ -761,7 +807,7 @@ class PyLinter(config.OptionsManagerMixIn,
         child_config = collections.OrderedDict()
         filter_options = {'long-help'}
         filter_options.update((opt_name for opt_name, _ in self._external_opts))
-        for opt_providers in six.itervalues(self._all_options):
+        for opt_providers in self._all_options.values():
             for optname, optdict, val in opt_providers.options_and_values():
                 if optdict.get('deprecated'):
                     continue
@@ -782,14 +828,20 @@ class PyLinter(config.OptionsManagerMixIn,
         tasks_queue = manager.Queue()
         results_queue = manager.Queue()
 
-        for _ in range(self.config.jobs):
+        # Send files to child linters.
+        expanded_files = []
+        for descr in self.expand_files(files_or_modules):
+            modname, filepath, is_arg = descr['name'], descr['path'], descr['isarg']
+            if self.should_analyze_file(modname, filepath, is_argument=is_arg):
+                expanded_files.append(descr)
+
+        # do not start more jobs than needed
+        for _ in range(min(self.config.jobs, len(expanded_files))):
             child_linter = ChildLinter(args=(tasks_queue, results_queue,
                                              child_config))
             child_linter.start()
             children.append(child_linter)
 
-        # Send files to child linters.
-        expanded_files = self.expand_files(files_or_modules)
         for files_or_module in expanded_files:
             path = files_or_module['path']
             tasks_queue.put([path])
@@ -814,7 +866,7 @@ class PyLinter(config.OptionsManagerMixIn,
             child.join()
 
         if failed:
-            print("Error occured, stopping the linter.", file=sys.stderr)
+            print("Error occurred, stopping the linter.", file=sys.stderr)
             sys.exit(32)
 
     def _parallel_check(self, files_or_modules):
@@ -824,6 +876,8 @@ class PyLinter(config.OptionsManagerMixIn,
         all_stats = []
         module = None
         for result in self._parallel_task(files_or_modules):
+            if not result:
+                continue
             (
                 _,
                 self.file_state.base_name,
@@ -864,12 +918,10 @@ class PyLinter(config.OptionsManagerMixIn,
                 walker.add_checker(checker)
         # build ast and check modules or packages
         for descr in self.expand_files(files_or_modules):
-            modname, filepath = descr['name'], descr['path']
-            if not descr['isarg'] and not self.should_analyze_file(modname, filepath):
+            modname, filepath, is_arg = descr['name'], descr['path'], descr['isarg']
+            if not self.should_analyze_file(modname, filepath, is_argument=is_arg):
                 continue
-            if self.config.files_output:
-                reportfile = 'pylint_%s.%s' % (modname, self.reporter.extension)
-                self.reporter.set_output(open(reportfile, 'w'))
+
             self.set_current_module(modname, filepath)
             # get the module representation
             ast_node = self.get_ast(filepath, modname)
@@ -918,22 +970,21 @@ class PyLinter(config.OptionsManagerMixIn,
         self.current_file = filepath or modname
         self.stats['by_module'][modname] = {}
         self.stats['by_module'][modname]['statement'] = 0
-        for msg_cat in six.itervalues(utils.MSG_TYPES):
+        for msg_cat in utils.MSG_TYPES.values():
             self.stats['by_module'][modname][msg_cat] = 0
 
     def get_ast(self, filepath, modname):
-        """return a ast(roid) representation for a module"""
+        """return an ast(roid) representation for a module"""
         try:
             return MANAGER.ast_from_file(filepath, modname, source=True)
+        except astroid.AstroidSyntaxError as ex:
+            # pylint: disable=no-member
+            self.add_message('syntax-error',
+                             line=getattr(ex.error, 'lineno', 0),
+                             args=str(ex.error))
         except astroid.AstroidBuildingException as ex:
-            if isinstance(ex.args[0], SyntaxError):
-                ex = ex.args[0]
-                self.add_message('syntax-error',
-                                 line=ex.lineno or 0,
-                                 args=ex.msg)
-            else:
-                self.add_message('parse-error', args=ex)
-        except Exception as ex: # pylint: disable=broad-except
+            self.add_message('parse-error', args=ex)
+        except Exception as ex:
             import traceback
             traceback.print_exc()
             self.add_message('astroid-error', args=(ex.__class__, ex))
@@ -945,7 +996,7 @@ class PyLinter(config.OptionsManagerMixIn,
             tokens = utils.tokenize_module(ast_node)
         except tokenize.TokenError as ex:
             self.add_message('syntax-error', line=ex.args[1][0], args=ex.args[0])
-            return
+            return None
 
         if not ast_node.pure_python:
             self.add_message('raw-checker-failed', args=ast_node.name)
@@ -974,11 +1025,11 @@ class PyLinter(config.OptionsManagerMixIn,
         self.stats = {'by_module' : {},
                       'by_msg' : {},
                      }
-        MANAGER.optimize_ast = self.config.optimize_ast
         MANAGER.always_load_extensions = self.config.unsafe_load_any_extension
+        MANAGER.max_inferable_values = self.config.limit_inference_results
         MANAGER.extension_package_whitelist.update(
             self.config.extension_pkg_whitelist)
-        for msg_cat in six.itervalues(utils.MSG_TYPES):
+        for msg_cat in utils.MSG_TYPES.values():
             self.stats[msg_cat] = 0
 
     def generate_reports(self):
@@ -996,40 +1047,42 @@ class PyLinter(config.OptionsManagerMixIn,
             self.reporter.on_close(self.stats, previous_stats)
             if self.config.reports:
                 sect = self.make_reports(self.stats, previous_stats)
-                if self.config.files_output:
-                    filename = 'pylint_global.' + self.reporter.extension
-                    self.reporter.set_output(open(filename, 'w'))
             else:
                 sect = report_nodes.Section()
+
             if self.config.reports:
                 self.reporter.display_reports(sect)
+            self._report_evaluation()
             # save results if persistent run
             if self.config.persistent:
                 config.save_results(self.stats, self.file_state.base_name)
         else:
             self.reporter.on_close(self.stats, {})
 
-    # specific reports ########################################################
-
-    def report_evaluation(self, sect, stats, previous_stats):
+    def _report_evaluation(self):
         """make the global evaluation report"""
         # check with at least check 1 statements (usually 0 when there is a
         # syntax error preventing pylint from further processing)
-        if stats['statement'] == 0:
-            raise utils.EmptyReport()
+        previous_stats = config.load_results(self.file_state.base_name)
+        if self.stats['statement'] == 0:
+            return
+
         # get a global note for the code
         evaluation = self.config.evaluation
         try:
             note = eval(evaluation, {}, self.stats) # pylint: disable=eval-used
-        except Exception as ex: # pylint: disable=broad-except
+        except Exception as ex:
             msg = 'An exception occurred while rating: %s' % ex
         else:
-            stats['global_note'] = note
+            self.stats['global_note'] = note
             msg = 'Your code has been rated at %.2f/10' % note
             pnote = previous_stats.get('global_note')
             if pnote is not None:
                 msg += ' (previous run: %.2f/10, %+.2f)' % (pnote, note - pnote)
-        sect.append(report_nodes.Text(msg))
+
+        if self.config.score:
+            sect = report_nodes.EvaluationSection(msg)
+            self.reporter.display_reports(sect)
 
 # some reporting functions ####################################################
 
@@ -1045,9 +1098,9 @@ def report_messages_stats(sect, stats, _):
     """make messages type report"""
     if not stats['by_msg']:
         # don't print this report when we didn't detected any errors
-        raise utils.EmptyReport()
+        raise exceptions.EmptyReportError()
     in_order = sorted([(value, msg_id)
-                       for msg_id, value in six.iteritems(stats['by_msg'])
+                       for msg_id, value in stats['by_msg'].items()
                        if not msg_id.startswith('I')])
     in_order.reverse()
     lines = ('message id', 'occurrences')
@@ -1059,11 +1112,11 @@ def report_messages_by_module_stats(sect, stats, _):
     """make errors / warnings by modules report"""
     if len(stats['by_module']) == 1:
         # don't print this report when we are analysing a single module
-        raise utils.EmptyReport()
+        raise exceptions.EmptyReportError()
     by_mod = collections.defaultdict(dict)
     for m_type in ('fatal', 'error', 'warning', 'refactor', 'convention'):
         total = stats[m_type]
-        for module in six.iterkeys(stats['by_module']):
+        for module in stats['by_module'].keys():
             mod_total = stats['by_module'][module][m_type]
             if total == 0:
                 percent = 0
@@ -1071,7 +1124,7 @@ def report_messages_by_module_stats(sect, stats, _):
                 percent = float((mod_total)*100) / total
             by_mod[module][m_type] = percent
     sorted_result = []
-    for module, mod_info in six.iteritems(by_mod):
+    for module, mod_info in by_mod.items():
         sorted_result.append((mod_info['error'],
                               mod_info['warning'],
                               mod_info['refactor'],
@@ -1088,7 +1141,7 @@ def report_messages_by_module_stats(sect, stats, _):
         for val in line[:-1]:
             lines.append('%.2f' % val)
     if len(lines) == 5:
-        raise utils.EmptyReport()
+        raise exceptions.EmptyReportError()
     sect.append(report_nodes.Table(children=lines, cols=5, rheaders=1))
 
 
@@ -1141,7 +1194,7 @@ def fix_import_path(args):
     Within this context, each of the given arguments is importable.
     Paths are added to sys.path in corresponding order to the arguments.
     We avoid adding duplicate directories to sys.path.
-    `sys.path` is reset to its original value upon exitign this context.
+    `sys.path` is reset to its original value upon exiting this context.
     """
     orig = list(sys.path)
     changes = []
@@ -1151,14 +1204,14 @@ def fix_import_path(args):
             continue
         else:
             changes.append(path)
-    sys.path[:] = changes + sys.path
+    sys.path[:] = changes + ["."] + sys.path
     try:
         yield
     finally:
         sys.path[:] = orig
 
 
-class Run(object):
+class Run:
     """helper class to use as main for pylint :
 
     run(*sys.argv[1:])
@@ -1169,15 +1222,17 @@ class Run(object):
 group are mutually exclusive.'),
         )
 
-    def __init__(self, args, reporter=None, exit=True):
+    def __init__(self, args, reporter=None, do_exit=True):
         self._rcfile = None
         self._plugins = []
+        self.verbose = None
         try:
             preprocess_options(args, {
                 # option: (callback, takearg)
                 'init-hook':   (cb_init_hook, True),
                 'rcfile':       (self.cb_set_rcfile, True),
                 'load-plugins': (self.cb_add_plugins, True),
+                'verbose': (self.cb_verbose_mode, False),
                 })
         except ArgumentPreprocessingError as ex:
             print(ex, file=sys.stderr)
@@ -1213,7 +1268,7 @@ group are mutually exclusive.'),
              {'action' : 'callback',
               'callback' : cb_list_confidence_levels,
               'group': 'Commands', 'level': 1,
-              'help' : "Generate pylint's messages."}),
+              'help' : "Generate pylint's confidence levels."}),
 
             ('full-documentation',
              {'action' : 'callback', 'metavar': '<msg-id>',
@@ -1239,13 +1294,19 @@ group are mutually exclusive.'),
               'short': 'E',
               'help' : 'In error mode, checkers without error messages are '
                        'disabled and for others, only the ERROR messages are '
-                       'displayed, and no reports are done by default'''}),
+                       'displayed, and no reports are done by default.'}),
 
             ('py3k',
              {'action' : 'callback', 'callback' : self.cb_python3_porting_mode,
               'help' : 'In Python 3 porting mode, all checkers will be '
                        'disabled and only messages emitted by the porting '
-                       'checker will be displayed'}),
+                       'checker will be displayed.'}),
+
+            ('verbose',
+             {'action' : 'callback', 'callback' : self.cb_verbose_mode,
+              'short': 'v',
+              'help' : 'In verbose mode, extra non-checker-related info '
+                       'will be displayed.'})
 
             ), option_groups=self.option_groups, pylintrc=self._rcfile)
         # register standard checkers
@@ -1282,9 +1343,9 @@ group are mutually exclusive.'),
 'been issued by analysing pylint output status code\n',
                                 level=1)
         # read configuration
-        linter.disable('suppressed-message')
-        linter.disable('useless-suppression')
-        linter.read_config_file()
+        linter.disable('I')
+        linter.enable('c-extension-no-member')
+        linter.read_config_file(verbose=self.verbose)
         config_parser = linter.cfgfile_parser
         # run init hook, if present, before loading plugins
         if config_parser.has_option('MASTER', 'init-hook'):
@@ -1315,7 +1376,7 @@ group are mutually exclusive.'),
             sys.exit(32)
 
         if linter.config.jobs < 0:
-            print("Jobs number (%d) should be greater than 0"
+            print("Jobs number (%d) should be greater than or equal to 0"
                   % linter.config.jobs, file=sys.stderr)
             sys.exit(32)
         if linter.config.jobs > 1 or linter.config.jobs == 0:
@@ -1332,8 +1393,11 @@ group are mutually exclusive.'),
         with fix_import_path(args):
             linter.check(args)
             linter.generate_reports()
-        if exit:
-            sys.exit(self.linter.msg_status)
+        if do_exit:
+            if linter.config.exit_zero:
+                sys.exit(0)
+            else:
+                sys.exit(self.linter.msg_status)
 
     def cb_set_rcfile(self, name, value):
         """callback for option preprocessing (i.e. before option parsing)"""
@@ -1383,6 +1447,8 @@ group are mutually exclusive.'),
         """Activate only the python3 porting checker."""
         self.linter.python3_porting_mode()
 
+    def cb_verbose_mode(self, *args, **kwargs):
+        self.verbose = True
 
 def cb_list_confidence_levels(option, optname, value, parser):
     for level in interfaces.CONFIDENCE_LEVELS:
