@@ -2,10 +2,13 @@
 # Copyright (c) 2012-2014 Google, Inc.
 # Copyright (c) 2014 Brett Cannon <brett@python.org>
 # Copyright (c) 2014 Arun Persaud <arun@nubati.net>
-# Copyright (c) 2015-2016 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2015-2018 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2015 Florian Bruhin <me@the-compiler.org>
 # Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
 # Copyright (c) 2016 y2kbugger <y2kbugger@users.noreply.github.com>
+# Copyright (c) 2018 Sushobhit <31987769+sushobhit27@users.noreply.github.com>
+# Copyright (c) 2018 Jace Browning <jacebrowning@gmail.com>
+# Copyright (c) 2018 Nick Drozd <nicholasdrozd@gmail.com>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
@@ -21,8 +24,6 @@ import os
 import warnings
 import sys
 
-import six
-
 from pylint.interfaces import IReporter
 from pylint.reporters import BaseReporter
 from pylint import utils
@@ -124,7 +125,7 @@ class TextReporter(BaseReporter):
     __implements__ = IReporter
     name = 'text'
     extension = 'txt'
-    line_format = '{C}:{line:3d},{column:2d}: {msg} ({symbol})'
+    line_format = '{path}:{line}:{column}: {msg_id}: {msg} ({symbol})'
 
     def __init__(self, output=None):
         BaseReporter.__init__(self, output)
@@ -132,7 +133,7 @@ class TextReporter(BaseReporter):
         self._template = None
 
     def on_set_current_module(self, module, filepath):
-        self._template = six.text_type(self.linter.config.msg_template or self.line_format)
+        self._template = str(self.linter.config.msg_template or self.line_format)
 
     def write_message(self, msg):
         """Convenience method to write a formated message with class default template"""
@@ -184,7 +185,7 @@ class ColorizedTextReporter(TextReporter):
         "I" : ("green", None),
         'C' : (None, "bold"),
         'R' : ("magenta", "bold, italic"),
-        'W' : ("blue", None),
+        'W' : ("magenta", None),
         'E' : ("red", "bold"),
         'F' : ("red", "bold, underline"),
         'S' : ("yellow", "inverse"), # S stands for module Separator
@@ -197,6 +198,7 @@ class ColorizedTextReporter(TextReporter):
         ansi_terms = ['xterm-16color', 'xterm-256color']
         if os.environ.get('TERM') not in ansi_terms:
             if sys.platform == 'win32':
+                # pylint: disable=import-error
                 import colorama
                 self.out = colorama.AnsiToWin32(self.out)
 
