@@ -1,20 +1,11 @@
-# copyright 2003-2015 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
-# contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
-#
-# This file is part of pylint.
-#
-# pylint is free software: you can redistribute it and/or modify it under
-# the terms of the GNU Lesser General Public License as published by the Free
-# Software Foundation, either version 2.1 of the License, or (at your option) any
-# later version.
-#
-# pylint is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
-# details.
-#
-# You should have received a copy of the GNU Lesser General Public License along
-# with pylint.  If not, see <http://www.gnu.org/licenses/>.
+# Copyright (c) 2015-2016 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2018 ssolanki <sushobhitsolanki@gmail.com>
+# Copyright (c) 2018 Sushobhit <31987769+sushobhit27@users.noreply.github.com>
+# Copyright (c) 2018 Anthony Sottile <asottile@umich.edu>
+
+# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+# For details: https://github.com/PyCQA/pylint/blob/master/COPYING
+
 """Universal report objects and some formatting drivers.
 
 A way to create simple reports using python objects, primarily designed to be
@@ -23,10 +14,10 @@ formatted as text and html.
 import os
 import sys
 
-import six
+from io import StringIO
 
 
-class BaseWriter(object):
+class BaseWriter:
     """base class for ureport writers"""
 
     def format(self, layout, stream=None, encoding=None):
@@ -53,7 +44,7 @@ class BaseWriter(object):
         for child in getattr(layout, 'children', ()):
             child.accept(self)
 
-    def writeln(self, string=u''):
+    def writeln(self, string=''):
         """write a line in the output buffer"""
         self.write(string + os.linesep)
 
@@ -83,7 +74,7 @@ class BaseWriter(object):
             result[-1].append(cell)
         # fill missing cells
         while len(result[-1]) < cols:
-            result[-1].append(u'')
+            result[-1].append('')
         return result
 
     def compute_content(self, layout):
@@ -98,7 +89,7 @@ class BaseWriter(object):
         out = self.out
         try:
             for child in layout.children:
-                stream = six.StringIO()
+                stream = StringIO()
                 self.out = stream
                 child.accept(self)
                 yield stream.getvalue()
