@@ -1,20 +1,26 @@
-"""Unit tests for the variables checker."""
-import unittest
-import sys
+# Copyright (c) 2014-2018 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2014 Google, Inc.
+# Copyright (c) 2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
+# Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
+# Copyright (c) 2016 Derek Gustafson <degustaf@gmail.com>
+# Copyright (c) 2018 ssolanki <sushobhitsolanki@gmail.com>
 
+# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+# For details: https://github.com/PyCQA/pylint/blob/master/COPYING
+
+"""Unit tests for the variables checker."""
 import astroid
-from astroid import test_utils
 from pylint.checkers import classes
 from pylint.testutils import CheckerTestCase, Message, set_config
 
-class VariablesCheckerTC(CheckerTestCase):
+class TestVariablesChecker(CheckerTestCase):
 
     CHECKER_CLASS = classes.ClassChecker
 
     def test_bitbucket_issue_164(self):
         """Issue 164 report a false negative for access-member-before-definition"""
-        n1, n2 = test_utils.extract_node("""
-        class MyClass1(object):
+        n1, n2 = astroid.extract_node("""
+        class MyClass1:
           def __init__(self):
             self.first += 5 #@
             self.first = 0  #@
@@ -31,7 +37,7 @@ class VariablesCheckerTC(CheckerTestCase):
         """
 
         node = astroid.parse("""
-        class Protected(object):
+        class Protected:
             '''empty'''
             def __init__(self):
                 self._meta = 42
@@ -48,13 +54,11 @@ class VariablesCheckerTC(CheckerTestCase):
                         args='_teta')):
             self.walk(node.root())
 
-    @unittest.skipUnless(sys.version_info[0] == 3,
-                         "The test works on Python 3.")
     def test_regression_non_parent_init_called_tracemalloc(self):
         # This used to raise a non-parent-init-called on Pylint 1.3
         # See issue https://bitbucket.org/logilab/pylint/issue/308/
         # for reference.
-        node = test_utils.extract_node("""
+        node = astroid.extract_node("""
         from tracemalloc import Sequence
         class _Traces(Sequence):
             def __init__(self, traces): #@
@@ -67,9 +71,9 @@ class VariablesCheckerTC(CheckerTestCase):
         # This should not emit a super-init-not-called
         # warning. It previously did this, because
         # ``next(node.infer())`` was used in that checker's
-        # logic and the first inferred node was an YES object,
+        # logic and the first inferred node was an Uninferable object,
         # leading to this false positive.
-        node = test_utils.extract_node("""
+        node = astroid.extract_node("""
         import ctypes
 
         class Foo(ctypes.BigEndianStructure):
@@ -78,7 +82,3 @@ class VariablesCheckerTC(CheckerTestCase):
         """)
         with self.assertNoMessages():
             self.checker.visit_functiondef(node)
-
-
-if __name__ == '__main__':
-    unittest.main()
